# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from .event import (
    EventResource,
    AsyncEventResource,
    EventResourceWithRawResponse,
    AsyncEventResourceWithRawResponse,
    EventResourceWithStreamingResponse,
    AsyncEventResourceWithStreamingResponse,
)
from .location import (
    LocationResource,
    AsyncLocationResource,
    LocationResourceWithRawResponse,
    AsyncLocationResourceWithRawResponse,
    LocationResourceWithStreamingResponse,
    AsyncLocationResourceWithStreamingResponse,
)
from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.skynet import (
    asset_bind_params,
    asset_list_params,
    asset_track_params,
    asset_create_params,
    asset_delete_params,
    asset_update_params,
    asset_retrieve_params,
    asset_update_attributes_params,
)
from ....types.skynet.simple_resp import SimpleResp
from ....types.skynet.meta_data_param import MetaDataParam
from ....types.skynet.asset_list_response import AssetListResponse
from ....types.skynet.asset_create_response import AssetCreateResponse
from ....types.skynet.asset_retrieve_response import AssetRetrieveResponse

__all__ = ["AssetResource", "AsyncAssetResource"]


class AssetResource(SyncAPIResource):
    @cached_property
    def event(self) -> EventResource:
        return EventResource(self._client)

    @cached_property
    def location(self) -> LocationResource:
        return LocationResource(self._client)

    @cached_property
    def with_raw_response(self) -> AssetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AssetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AssetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AssetResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        attributes: object | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        meta_data: MetaDataParam | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetCreateResponse:
        """
        Create an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          attributes: `attributes` can be used to store custom information about an asset in
              `key`:`value` format. Use `attributes` to add any useful information or context
              to your assets like the vehicle type, shift timing etc. Moreover, these
              attributes can be used to filter `assets` in **Search**, **Monitor**, and _Get
              Asset List_ queries.

              Please note that the maximum number of `key`:`value` pairs that can be added to
              an `attributes` object is 100. Also, the overall size of `attributes` object
              should not exceed 65kb.

          custom_id: Set a unique ID for the new `asset`. If not provided, an ID will be
              automatically generated in UUID format. A valid `custom_id` can contain letters,
              numbers, "-", & "\\__" only.

              Please note that the ID of an `asset` can not be changed once it is created.

          description: Description for the `asset`.

          meta_data: Any valid json object data. Can be used to save customized data. Max size is
              65kb.

          name: Name of the `asset`. Use this field to assign a meaningful, custom name to the
              `asset` being created.

          tags: **This parameter will be deprecated soon! Please use the `attributes` parameter
              to add labels or markers for the asset.**

              Tags of the `asset`. `tags` can be used for filtering assets in operations like
              _Get Asset List_ and asset **Search** methods. They can also be used for
              monitoring of assets using the **Monitor** methods after linking `tags` and
              `asset`.

              Valid `tags` are strings consisting of alphanumeric characters (A-Z, a-z, 0-9)
              along with the underscore ('\\__') and hyphen ('-') symbols.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/skynet/asset",
            body=maybe_transform(
                {
                    "attributes": attributes,
                    "custom_id": custom_id,
                    "description": description,
                    "meta_data": meta_data,
                    "name": name,
                    "tags": tags,
                },
                asset_create_params.AssetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_create_params.AssetCreateParams,
                ),
            ),
            cast_to=AssetCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetRetrieveResponse:
        """
        Get an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/skynet/asset/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_retrieve_params.AssetRetrieveParams,
                ),
            ),
            cast_to=AssetRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        attributes: object | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        meta_data: MetaDataParam | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Update an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          attributes: Use this param to update the `attributes` of an asset in `key`:`value` format.
              Users can maintain any useful information or context about the assets by
              utilising this parameter.

              Please be careful when using this parameter while updating an `asset` as the new
              `attributes` object provided will completely overwrite the old `attributes`
              object. Use the _Update Asset Attributes_ method to add new or modify existing
              attributes.

              Another point to note is that the overall size of the `attributes` object cannot
              exceed 65kb and the maximum number of `key`:`value` pairs that can be added to
              this object is 100.

          description: Use this param to update the `description` of an `asset`.

          meta_data: Any valid json object data. Can be used to save customized data. Max size is
              65kb.

          name: Use this param to update the `name` of an `asset`. Users can assign meaningful
              custom names to their assets.

          tags: **This parameter will be deprecated soon! Please use the `attributes` parameter
              to add labels or markers for the asset.**

              Use this param to update the `tags` of an `asset`. `tags` can be used to filter
              `asset` in _Get Asset List_, **Search** and **Monitor** queries.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/skynet/asset/{id}",
            body=maybe_transform(
                {
                    "attributes": attributes,
                    "description": description,
                    "meta_data": meta_data,
                    "name": name,
                    "tags": tags,
                },
                asset_update_params.AssetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_update_params.AssetUpdateParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    def list(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        include_all_of_attributes: str | NotGiven = NOT_GIVEN,
        include_any_of_attributes: str | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        sort: str | NotGiven = NOT_GIVEN,
        tags: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetListResponse:
        """
        Get Asset List

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          include_all_of_attributes: Use this parameter to filter the assets by their `attributes`. Only the assets
              having all the `attributes` added to this parameter, will be returned in the
              response. Multiple `attributes` can be separated using pipes (`|`).

              Please note the attributes are case sensitive. Also, this parameter can not be
              used in conjunction with `include_any_of_attributes` parameter.

          include_any_of_attributes: Use this parameter to filter the assets by their `attributes`. Assets having at
              least one of the `attributes` added to this parameter, will be returned in the
              response. Multiple `attributes` can be separated using pipes (`|`).

              Please note the attributes are case sensitive. Also, this parameter can not be
              used in conjunction with `include_all_of_attributes` parameter.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          sort: Provide a single field to sort the results by. Only `updated_at` or `created_at`
              fields can be selected for ordering the results.

              By default, the result is sorted by `created_at` field in the descending order.
              Allowed values for specifying the order are `asc` for ascending order and `desc`
              for descending order.

          tags: **This parameter will be deprecated soon! Please use the
              `include_all_of_attributes` or `include_any_of_attributes` parameters to provide
              labels or markers for the assets to be retrieved.**

              `tags` can be used to filter the assets. Only those assets which have all the
              `tags` provided, will be included in the result. In case multiple `tags` need to
              be specified, use `,` to separate them.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/skynet/asset/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                        "include_all_of_attributes": include_all_of_attributes,
                        "include_any_of_attributes": include_any_of_attributes,
                        "pn": pn,
                        "ps": ps,
                        "sort": sort,
                        "tags": tags,
                    },
                    asset_list_params.AssetListParams,
                ),
            ),
            cast_to=AssetListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Delete an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/skynet/asset/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_delete_params.AssetDeleteParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    def bind(
        self,
        id: str,
        *,
        key: str,
        device_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Bind asset to device

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          device_id: Device ID to be linked to the `asset` identified by `id`.

              Please note that the device needs to be linked to an `asset` before using it in
              the _Upload locations of an Asset_ method for sending GPS information about the
              `asset`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/skynet/asset/{id}/bind",
            body=maybe_transform({"device_id": device_id}, asset_bind_params.AssetBindParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, asset_bind_params.AssetBindParams),
            ),
            cast_to=SimpleResp,
        )

    def track(
        self,
        id: str,
        *,
        key: str,
        device_id: str,
        locations: asset_track_params.Locations,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Upload track info

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          device_id: ID of the device used to upload the tracking information of the `asset`.

              Please note that the `device_id` used here must already be linked to the
              `asset`. Use the _Bind Device to Asset_ method to link a device with your
              `asset`.

          locations: An array of objects to collect the location tracking information for an `asset`.
              Each object must correspond to details of only one location.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/skynet/asset/{id}/track",
            body=maybe_transform(
                {
                    "device_id": device_id,
                    "locations": locations,
                },
                asset_track_params.AssetTrackParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_track_params.AssetTrackParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    def update_attributes(
        self,
        id: str,
        *,
        key: str,
        attributes: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """Update asset attributes.

        (add)

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          attributes: `attributes` can be used to add any useful information or context to your assets
              like the vehicle type, shift timing etc. These attributes can also be used to
              filter assets in **Search**, **Monitor**, and _Get Asset List_ queries.

              Provide the attributes to be added or updated, in `key`:`value` format. If an
              existing `key` is provided in the input, then the `value` will be modified as
              per the input value. If a new `key` is provided in the input, then the `key`
              would be added to the existing set. The contents of any `value` field are
              neither altered nor removed unless specifically referred to by its `key` in the
              input request.

              Please note that the maximum number of `key`:`value` pairs that can be added to
              an `attributes` object is 100. Also, the overall size of `attributes` object
              should not exceed 65kb.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/skynet/asset/{id}/attributes",
            body=maybe_transform(
                {"attributes": attributes}, asset_update_attributes_params.AssetUpdateAttributesParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, asset_update_attributes_params.AssetUpdateAttributesParams),
            ),
            cast_to=SimpleResp,
        )


class AsyncAssetResource(AsyncAPIResource):
    @cached_property
    def event(self) -> AsyncEventResource:
        return AsyncEventResource(self._client)

    @cached_property
    def location(self) -> AsyncLocationResource:
        return AsyncLocationResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAssetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAssetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAssetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncAssetResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        attributes: object | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        meta_data: MetaDataParam | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetCreateResponse:
        """
        Create an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          attributes: `attributes` can be used to store custom information about an asset in
              `key`:`value` format. Use `attributes` to add any useful information or context
              to your assets like the vehicle type, shift timing etc. Moreover, these
              attributes can be used to filter `assets` in **Search**, **Monitor**, and _Get
              Asset List_ queries.

              Please note that the maximum number of `key`:`value` pairs that can be added to
              an `attributes` object is 100. Also, the overall size of `attributes` object
              should not exceed 65kb.

          custom_id: Set a unique ID for the new `asset`. If not provided, an ID will be
              automatically generated in UUID format. A valid `custom_id` can contain letters,
              numbers, "-", & "\\__" only.

              Please note that the ID of an `asset` can not be changed once it is created.

          description: Description for the `asset`.

          meta_data: Any valid json object data. Can be used to save customized data. Max size is
              65kb.

          name: Name of the `asset`. Use this field to assign a meaningful, custom name to the
              `asset` being created.

          tags: **This parameter will be deprecated soon! Please use the `attributes` parameter
              to add labels or markers for the asset.**

              Tags of the `asset`. `tags` can be used for filtering assets in operations like
              _Get Asset List_ and asset **Search** methods. They can also be used for
              monitoring of assets using the **Monitor** methods after linking `tags` and
              `asset`.

              Valid `tags` are strings consisting of alphanumeric characters (A-Z, a-z, 0-9)
              along with the underscore ('\\__') and hyphen ('-') symbols.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/skynet/asset",
            body=await async_maybe_transform(
                {
                    "attributes": attributes,
                    "custom_id": custom_id,
                    "description": description,
                    "meta_data": meta_data,
                    "name": name,
                    "tags": tags,
                },
                asset_create_params.AssetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_create_params.AssetCreateParams,
                ),
            ),
            cast_to=AssetCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetRetrieveResponse:
        """
        Get an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/skynet/asset/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_retrieve_params.AssetRetrieveParams,
                ),
            ),
            cast_to=AssetRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        attributes: object | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        meta_data: MetaDataParam | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Update an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          attributes: Use this param to update the `attributes` of an asset in `key`:`value` format.
              Users can maintain any useful information or context about the assets by
              utilising this parameter.

              Please be careful when using this parameter while updating an `asset` as the new
              `attributes` object provided will completely overwrite the old `attributes`
              object. Use the _Update Asset Attributes_ method to add new or modify existing
              attributes.

              Another point to note is that the overall size of the `attributes` object cannot
              exceed 65kb and the maximum number of `key`:`value` pairs that can be added to
              this object is 100.

          description: Use this param to update the `description` of an `asset`.

          meta_data: Any valid json object data. Can be used to save customized data. Max size is
              65kb.

          name: Use this param to update the `name` of an `asset`. Users can assign meaningful
              custom names to their assets.

          tags: **This parameter will be deprecated soon! Please use the `attributes` parameter
              to add labels or markers for the asset.**

              Use this param to update the `tags` of an `asset`. `tags` can be used to filter
              `asset` in _Get Asset List_, **Search** and **Monitor** queries.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/skynet/asset/{id}",
            body=await async_maybe_transform(
                {
                    "attributes": attributes,
                    "description": description,
                    "meta_data": meta_data,
                    "name": name,
                    "tags": tags,
                },
                asset_update_params.AssetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_update_params.AssetUpdateParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    async def list(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        include_all_of_attributes: str | NotGiven = NOT_GIVEN,
        include_any_of_attributes: str | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        sort: str | NotGiven = NOT_GIVEN,
        tags: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetListResponse:
        """
        Get Asset List

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          include_all_of_attributes: Use this parameter to filter the assets by their `attributes`. Only the assets
              having all the `attributes` added to this parameter, will be returned in the
              response. Multiple `attributes` can be separated using pipes (`|`).

              Please note the attributes are case sensitive. Also, this parameter can not be
              used in conjunction with `include_any_of_attributes` parameter.

          include_any_of_attributes: Use this parameter to filter the assets by their `attributes`. Assets having at
              least one of the `attributes` added to this parameter, will be returned in the
              response. Multiple `attributes` can be separated using pipes (`|`).

              Please note the attributes are case sensitive. Also, this parameter can not be
              used in conjunction with `include_all_of_attributes` parameter.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          sort: Provide a single field to sort the results by. Only `updated_at` or `created_at`
              fields can be selected for ordering the results.

              By default, the result is sorted by `created_at` field in the descending order.
              Allowed values for specifying the order are `asc` for ascending order and `desc`
              for descending order.

          tags: **This parameter will be deprecated soon! Please use the
              `include_all_of_attributes` or `include_any_of_attributes` parameters to provide
              labels or markers for the assets to be retrieved.**

              `tags` can be used to filter the assets. Only those assets which have all the
              `tags` provided, will be included in the result. In case multiple `tags` need to
              be specified, use `,` to separate them.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/skynet/asset/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                        "include_all_of_attributes": include_all_of_attributes,
                        "include_any_of_attributes": include_any_of_attributes,
                        "pn": pn,
                        "ps": ps,
                        "sort": sort,
                        "tags": tags,
                    },
                    asset_list_params.AssetListParams,
                ),
            ),
            cast_to=AssetListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Delete an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/skynet/asset/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_delete_params.AssetDeleteParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    async def bind(
        self,
        id: str,
        *,
        key: str,
        device_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Bind asset to device

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          device_id: Device ID to be linked to the `asset` identified by `id`.

              Please note that the device needs to be linked to an `asset` before using it in
              the _Upload locations of an Asset_ method for sending GPS information about the
              `asset`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/skynet/asset/{id}/bind",
            body=await async_maybe_transform({"device_id": device_id}, asset_bind_params.AssetBindParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, asset_bind_params.AssetBindParams),
            ),
            cast_to=SimpleResp,
        )

    async def track(
        self,
        id: str,
        *,
        key: str,
        device_id: str,
        locations: asset_track_params.Locations,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Upload track info

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          device_id: ID of the device used to upload the tracking information of the `asset`.

              Please note that the `device_id` used here must already be linked to the
              `asset`. Use the _Bind Device to Asset_ method to link a device with your
              `asset`.

          locations: An array of objects to collect the location tracking information for an `asset`.
              Each object must correspond to details of only one location.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/skynet/asset/{id}/track",
            body=await async_maybe_transform(
                {
                    "device_id": device_id,
                    "locations": locations,
                },
                asset_track_params.AssetTrackParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    asset_track_params.AssetTrackParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    async def update_attributes(
        self,
        id: str,
        *,
        key: str,
        attributes: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """Update asset attributes.

        (add)

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          attributes: `attributes` can be used to add any useful information or context to your assets
              like the vehicle type, shift timing etc. These attributes can also be used to
              filter assets in **Search**, **Monitor**, and _Get Asset List_ queries.

              Provide the attributes to be added or updated, in `key`:`value` format. If an
              existing `key` is provided in the input, then the `value` will be modified as
              per the input value. If a new `key` is provided in the input, then the `key`
              would be added to the existing set. The contents of any `value` field are
              neither altered nor removed unless specifically referred to by its `key` in the
              input request.

              Please note that the maximum number of `key`:`value` pairs that can be added to
              an `attributes` object is 100. Also, the overall size of `attributes` object
              should not exceed 65kb.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/skynet/asset/{id}/attributes",
            body=await async_maybe_transform(
                {"attributes": attributes}, asset_update_attributes_params.AssetUpdateAttributesParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"key": key}, asset_update_attributes_params.AssetUpdateAttributesParams
                ),
            ),
            cast_to=SimpleResp,
        )


class AssetResourceWithRawResponse:
    def __init__(self, asset: AssetResource) -> None:
        self._asset = asset

        self.create = to_raw_response_wrapper(
            asset.create,
        )
        self.retrieve = to_raw_response_wrapper(
            asset.retrieve,
        )
        self.update = to_raw_response_wrapper(
            asset.update,
        )
        self.list = to_raw_response_wrapper(
            asset.list,
        )
        self.delete = to_raw_response_wrapper(
            asset.delete,
        )
        self.bind = to_raw_response_wrapper(
            asset.bind,
        )
        self.track = to_raw_response_wrapper(
            asset.track,
        )
        self.update_attributes = to_raw_response_wrapper(
            asset.update_attributes,
        )

    @cached_property
    def event(self) -> EventResourceWithRawResponse:
        return EventResourceWithRawResponse(self._asset.event)

    @cached_property
    def location(self) -> LocationResourceWithRawResponse:
        return LocationResourceWithRawResponse(self._asset.location)


class AsyncAssetResourceWithRawResponse:
    def __init__(self, asset: AsyncAssetResource) -> None:
        self._asset = asset

        self.create = async_to_raw_response_wrapper(
            asset.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            asset.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            asset.update,
        )
        self.list = async_to_raw_response_wrapper(
            asset.list,
        )
        self.delete = async_to_raw_response_wrapper(
            asset.delete,
        )
        self.bind = async_to_raw_response_wrapper(
            asset.bind,
        )
        self.track = async_to_raw_response_wrapper(
            asset.track,
        )
        self.update_attributes = async_to_raw_response_wrapper(
            asset.update_attributes,
        )

    @cached_property
    def event(self) -> AsyncEventResourceWithRawResponse:
        return AsyncEventResourceWithRawResponse(self._asset.event)

    @cached_property
    def location(self) -> AsyncLocationResourceWithRawResponse:
        return AsyncLocationResourceWithRawResponse(self._asset.location)


class AssetResourceWithStreamingResponse:
    def __init__(self, asset: AssetResource) -> None:
        self._asset = asset

        self.create = to_streamed_response_wrapper(
            asset.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            asset.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            asset.update,
        )
        self.list = to_streamed_response_wrapper(
            asset.list,
        )
        self.delete = to_streamed_response_wrapper(
            asset.delete,
        )
        self.bind = to_streamed_response_wrapper(
            asset.bind,
        )
        self.track = to_streamed_response_wrapper(
            asset.track,
        )
        self.update_attributes = to_streamed_response_wrapper(
            asset.update_attributes,
        )

    @cached_property
    def event(self) -> EventResourceWithStreamingResponse:
        return EventResourceWithStreamingResponse(self._asset.event)

    @cached_property
    def location(self) -> LocationResourceWithStreamingResponse:
        return LocationResourceWithStreamingResponse(self._asset.location)


class AsyncAssetResourceWithStreamingResponse:
    def __init__(self, asset: AsyncAssetResource) -> None:
        self._asset = asset

        self.create = async_to_streamed_response_wrapper(
            asset.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            asset.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            asset.update,
        )
        self.list = async_to_streamed_response_wrapper(
            asset.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            asset.delete,
        )
        self.bind = async_to_streamed_response_wrapper(
            asset.bind,
        )
        self.track = async_to_streamed_response_wrapper(
            asset.track,
        )
        self.update_attributes = async_to_streamed_response_wrapper(
            asset.update_attributes,
        )

    @cached_property
    def event(self) -> AsyncEventResourceWithStreamingResponse:
        return AsyncEventResourceWithStreamingResponse(self._asset.event)

    @cached_property
    def location(self) -> AsyncLocationResourceWithStreamingResponse:
        return AsyncLocationResourceWithStreamingResponse(self._asset.location)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MdmCreateDistanceMatrixParams"]


class MdmCreateDistanceMatrixParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    option: Required[Literal["flexible"]]
    """
    Use this option to switch to truck-specific routing or time based routing or if
    you want to choose between the fastest and shortest route types.
    """

    origins: Required[str]
    """`origins` are the starting point of your route.

    Ensure that origins are routable land locations. Multiple origins should be
    separated by a pipe symbol (|).

    **Format:** latitude_1,longitude_1|latitude_2,longitude_2|…
    """

    spliter: Literal["od_number_spliter", "straight_distance_spliter", "location_spliter"]
    """Specify a spliter to split the matrix by. It accepts 2 values:

    - `od_number_spliter`:

    - `straight_distance_spliter`:

    Please note it is an internal, debug field only.

    _debug field. choose specific spliter to split matrix._
    """

    area: Literal["singapore", "usa", "india"]
    """Provide the country that the coordinates belong to.

    _the input coordinates area._
    """

    avoid: Literal[
        "`toll`",
        "`ferry`",
        "`highway`",
        "`sharp_turn`",
        "`service_road`",
        "`bbox`",
        "`left_turn`",
        "`right_turn`",
        "`none`",
    ]
    """Setting this will ensure the route avoids the object(s) specified as input.

    Multiple values should be separated by a pipe (|). If `none` is provided along
    with other values, an error is returned as a valid route is not feasible.

    - **Note:**

      - This parameter is effective only when `route_type=fastest`.
      - When this parameter is not provided in the input, ferries are set to be
        avoided by default. When `avoid` input is provided, only the mentioned
        objects are avoided.
      - When using `avoid=bbox` users also need to specify the boundaries of the
        bounding box to be avoid. Multiple bounding boxes can be specified
        simultaneously. Please note that bounding box is a hard filter and if it
        blocks all possible routes between given locations, a 4xx error is returned.

        - **Format:** bbox: min_latitude,min_longtitude,max_latitude,max_longitude.
        - **Example:** avoid=bbox: 34.0635,-118.2547, 34.0679,-118.2478 | bbox:
          34.0521,-118.2342, 34.0478,-118.2437

      - When using `avoid=sharp_turn`, default range of permissible turn angles is
        \\[[120,240\\]].
    """

    cross_border: bool
    """
    Specify if crossing an international border is expected for operations near
    border areas. When set to false, the API will prohibit routes going back & forth
    between countries. Consequently, routes within the same country will be
    preferred if they are feasible for the given set of `destination` or `waypoints`
    . When set to true, the routes will be allowed to go back & forth between
    countries as needed.

    This feature is available in North America region only. Please get in touch with
    [support@nextbillion.ai](mailto:support@nextbillion.ai) to enquire/enable other
    areas.
    """

    departure_time: int
    """
    This is a number in UNIX epoch timestamp in seconds format that can be used to
    provide the departure time. The response will return the `distance` and
    `duration` of the route based on typical traffic for at the given start time.If
    no input is provided for this parameter then the traffic conditions at the time
    of making the request are considered.

    Please note that when `route_type` is set to `shortest` then the
    `departure_time` will be ineffective as the service will return the result for
    the shortest path possible irrespective of the traffic conditions.
    """

    destinations: str
    """`destinations` are the ending coordinates of your route.

    Ensure that destinations are routable land locations. Multiple destinations
    should be separated by a pipe symbol (|).

    In case `destinations` are not provided or if it is left empty, then the input
    value of `origins` will be copied to `destinations` to create the OD matrix
    pairs.

    **Format:** latitude_1,longitude_1|latitude_2,longitude_2|…
    """

    destinations_approach: Literal["`unrestricted`", "`curb`"]
    """Specify the side of the road from which to approach `destinations` points.

    Please note that the given approach will be applied to all the `destinations`.
    """

    hazmat_type: Literal["`general`", "`circumstantial`", "`explosive`", "`harmful_to_water`"]
    """
    Specify the type of hazardous material being carried and the service will avoid
    roads which are not suitable for the type of goods specified. Multiple values
    can be separated using a pipe operator `|` .

    Please note that this parameter is effective only when `mode=truck`.
    """

    mode: Literal["`car`", "`truck`"]
    """Set which driving mode the service should use to determine a route.

    For example, if you use `car`, the API will return a route that a car can take.
    Using `truck` will return a route a truck can use, taking into account
    appropriate truck routing restrictions.
    """

    origins_approach: Literal["`unrestricted`", "`curb`"]
    """Specify the side of the road from which to approach `origins` points.

    Please note that the given approach will be applied to all the points provided
    as `origins`.
    """

    route_type: Literal["`fastest`", "`shortest`"]
    """Set the route type that needs to be returned.

    Please note that `route_type` is effective only when `option=flexible`.
    """

    truck_axle_load: float
    """
    Specify the total load per axle (including the weight of trailers and shipped
    goods) of the truck, in tonnes. When used, the service will return routes which
    are legally allowed to carry the load specified per axle.

    Please note this parameter is effective only when `mode=truck`.
    """

    truck_size: str
    """
    This defines the dimensions of a truck in centimeters (cm) in the format of
    "height,width,length". This parameter is effective only when `mode=truck` and
    `option=flexible`. Maximum dimensions are as follows:

    Height = 1000 cm Width = 5000 cm Length = 5000 cm
    """

    truck_weight: int
    """
    This parameter defines the weight of the truck including trailers and shipped
    goods in kilograms (kg). This parameter is effective only when `mode=truck` and
    `option=flexible`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from .metadata_param import MetadataParam

__all__ = ["MonitorUpdateParams", "GeofenceConfig", "IdleConfig", "MatchFilter", "SpeedingConfig"]


class MonitorUpdateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    description: str
    """Use this parameter to update the `description` of the `monitor`."""

    geofence_config: GeofenceConfig
    """
    `geofence_config` is used to update the set of geofences linked to the `monitor`
    for creating `enter` or `exit` type of events based on the asset's location.
    Please note that this object is mandatory when the monitor `type` belongs to one
    of `enter`, `exit` or `enter_and_exit`.
    """

    geofence_ids: List[str]
    """
    Use this parameter to update the geofences linked to the `monitor` by providing
    the geofence `id` as `,` separated strings. Geofences are geographic boundaries
    that can be used to trigger events based on an asset's location.
    """

    idle_config: IdleConfig
    """`idle_config` is used to update the constraints for creating idle events.

    When an asset associated with the `monitor` has not moved a given distance
    within a given time, the Live Tracking API can create events to denote such
    instances.

    Please note that this object is mandatory when the monitor `type` is `idle`.
    """

    match_filter: MatchFilter
    """Use this object to update the `attributes` of the `monitor`.

    Please note that using this property will overwrite the existing `attributes`
    that the monitor might be using currently to match any asset(s).
    """

    meta_data: MetadataParam
    """Any valid json object data.

    Can be used to save customized data. Max size is 65kb.
    """

    name: str
    """Use this parameter to update the `name` of the `monitor`.

    Users can add meaningful names to the monitors like "warehouse_exit",
    "depot_entry" etc.
    """

    speeding_config: SpeedingConfig
    """
    `speeding_config` is used to update the tolerance values for creating over-speed
    events. When an asset associated with a `monitor` is traveling at a speed above
    the given limits, Live Tracking API creates events to indicate such instances.

    Please note that this object is mandatory when the monitor `type` is `speeding`.
    """

    tags: List[str]
    """Use this parameter to update the `tags` of the `monitor`.

    `tags` can be used for filtering monitors in the _Get Monitor List_ operation.
    They can also be used for easy identification of monitors. Using this parameter
    overwrites the existing `tags` of the monitor.

    Please note that valid `tags` are strings, consisting of alphanumeric characters
    (A-Z, a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.
    """

    type: Literal["`enter`", "`exit`", "`enter_and_exit`", "`speeding`", "`idle`"]
    """Use this parameter to update the `type` of the `monitor`.

    The `monitor` will be able to detect the specified `type` of activity and create
    events for eligible `asset`. A `monitor` can detect following types of asset
    activity:

    - `enter`: The `monitor` will create an event when a linked `asset` enters into
      the specified geofence.

    - `exit`: The `monitor` will create an event when a linked `asset` exits the
      specified geofence.

    - `enter_and_exit`: The `monitor` will create an event when a linked `asset`
      either enters or exits the specified geofence.

    - `speeding`: The `monitor` will create an event when a linked `asset` exceeds a
      given speed limit.

    - `idle`: The `monitor` will create an event when a linked `asset` exhibits idle
      activity.

    Please note that `assets` and geofences can be linked to a `monitor` using the
    `match_filter` and `geofence_config` attributes respectively.
    """


class GeofenceConfig(TypedDict, total=False):
    geofence_ids: Required[List[str]]
    """Use this array to update the geofence IDs that should be linked to the
    `monitor`.

    Please note `geofence_ids` are mandatory when using the `geofence_config`
    attribute.
    """


class IdleConfig(TypedDict, total=False):
    distance_tolerance: Required[float]
    """
    Use this parameter to update the distance threshold that will be used to
    determine if the asset was idle or not. When the asset, within `time_tolerance`
    duration, moves less than the value for this parameter, the `monitor` creates an
    idle event against the `asset`. The `distance_tolerance` should be provided in
    meters.

    Please note `distance_tolerance` is mandatory when `idle_config` attribute is
    used.
    """

    time_tolerance: int
    """
    Use this parameter to update the time duration for which the `monitor` would
    track the distance covered by an asset before triggering an idle event. The
    `time_tolerance` should be provided in milliseconds.

    If the distance covered by the asset during a `time_tolerance` is less than that
    specified in `distance_tolerance` the asset will be assumed to be idle.

    This attribute along with `distance_tolerance` parameter can be used to control
    the "sensitivity" of the `monitor` with respect to idle alerts. It is
    recommended to use these properties with appropriate values to trigger genuine
    idle events. The appropriate values might depend on the traffic conditions,
    nature of operations that the asset is involved in, type of asset and other
    factors.
    """


class MatchFilter(TypedDict, total=False):
    include_all_of_attributes: object
    """A string type dictionary object to specify the `attributes`.

    Only the assets having all of the `attributes` added to this parameter will be
    linked to this `monitor`. Once an `asset` is linked to a `monitor`, the
    `monitor` will be able to create events for that `asset` whenever an activity
    specified in `type` is detected. Multiple attributes should be separated by a
    comma `,`.

    Please note that this parameter can not be used in conjunction with
    `include_any_of_attributes`. Also, the maximum number of `key`:`value` pairs
    that this parameter can take is 100 and the overall size of the `match_filter`
    object should not exceed 65kb.
    """

    include_any_of_attributes: object
    """A string type dictionary object to specify the `attributes`.

    The assets having at least one of the `attributes` added to this parameter will
    be linked to this `monitor`. Once an `asset` is linked to a `monitor`, the
    `monitor` will be able to create events for that `asset` whenever an activity
    specified in `type` is detected. Multiple attributes should be separated by a
    comma `,`.

    Please note that this parameter can not be used in conjunction with
    `include_all_of_attributes`. Also, the maximum number of `key`:`value` pairs
    that this parameter can take is 100 and the overall size of the `match_filter`
    object should not exceed 65kb.
    """


class SpeedingConfig(TypedDict, total=False):
    customer_speed_limit: str
    """
    Use this parameter to update the speed limit value that the `monitor` will use
    to create events, depending on the `time_tolerance` value. The speed limit
    should be specified in meters per second.

    Please note that `customer_speed_limit` is mandatory when
    `use_admin_speed_limit` is false. However, when `use_admin_speed_limit` is true,
    `customer_speed_limit` is ineffective.
    """

    time_tolerance: int
    """Use this parameter to update the time tolerance before triggering an event.

    Adding a tolerance would make the Tracking service wait for the specified time
    before triggering the event. Consequently, an event is triggered only when the
    time for which the asset has been over-speeding continuously, exceeds the
    configured tolerance time. The unit for this parameter is milliseconds.

    It can be seen that this attribute is used to control the "sensitivity" of the
    `monitor` with respect to speed alerts. Higher the value of `time_tolerance` the
    less sensitive the `monitor` would be to instances of over-speeding. Conversely,
    if 'time_tolerance' is set to 0, the `monitor` will be extremely sensitive and
    will create an event as soon as tracking information with a speed value greater
    than the specified limit is received.
    """

    use_admin_speed_limit: bool
    """
    Use this attribute to update which speed limit values will be used by the
    `monitor`. When `use_admin_speed_limit` is true, the administrative speed limit
    of the road on which the asset is located, is used to generate events when the
    asset’s tracked speed exceeds it. Whereas, when `use_admin_speed_limit` is
    false, the `customer_speed_limit` specified will be used to generate events when
    the asset's tracked speed exceeds it.

    Please note that if `use_admin_speed_limit` is false, `customer_speed_limit` is
    mandatory, otherwise when `use_admin_speed_limit` is true then
    `customer_speed_limit` is ineffective.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["RestrictionDeleteResponse"]


class RestrictionDeleteResponse(BaseModel):
    id: Optional[float] = None
    """It is the unique ID of the restriction."""

    state: Optional[str] = None
    """Returns the state of the restriction. It would always be `deleted`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["EventListParams"]


class EventListParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    end_time: int
    """Time before which the events triggered by the `asset` need to be retrieved."""

    monitor_id: str
    """Filter the events by `monitor_id`.

    When provided, only the events triggered by the `monitor` will be returned in
    response.

    Please note that if the `attributes` of the asset identified by `id` and those
    of the `monitor` do not match, then no events might be returned for this
    `monitor_id`.
    """

    pn: int
    """Denotes page number.

    Use this along with the `ps` parameter to implement pagination for your searched
    results. This parameter does not have a maximum limit but would return an empty
    response in case a higher value is provided when the result-set itself is
    smaller.
    """

    ps: int
    """Denotes number of search results per page.

    Use this along with the `pn` parameter to implement pagination for your searched
    results.
    """

    start_time: int
    """Time after which the events triggered by the `asset` need to be retrieved."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["BatchCreateParams", "Request"]


class BatchCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    requests: Iterable[Request]
    """
    An array of objects to collect the details of individual routing queries that
    will form a batch.
    """


class Request(TypedDict, total=False):
    query: str
    """Specify the routing query in the form of a string.

    The supported attributes and their formats are consistent with the standard
    routing endpoint that is being used as part of the batch. Check the
    [Sample Request](https://docs.nextbillion.ai/docs/navigation/batch-routing-api#sample-request-1)
    section for an example request.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["MdmGetDistanceMatrixStatusResponse"]


class MdmGetDistanceMatrixStatusResponse(BaseModel):
    code: Optional[Literal["`Ok`", "`Processing`", "`Failed`"]] = None
    """A code representing the status of the request."""

    output_addr: Optional[str] = None
    """Returns the GCS result of a successful task.

    Please note that this is an internal field.

    _internal field, the gcs result of specific task if task is success._
    """

    result_link: Optional[str] = None
    """
    Returns the link for the result file (csv format) once the task is completed
    successfully.
    """

    status: Optional[str] = None
    """Returns the status detail of the result.

    Indicative error messages/codes are returned in case of errors. See the
    [API Error Codes](#api-error-codes) section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["LocationParam"]


class LocationParam(TypedDict, total=False):
    lat: Required[float]
    """Latitude of location."""

    lon: Required[float]
    """Longitude of location."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["TripStartParams", "Stop"]


class TripStartParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    asset_id: Required[str]
    """Specify the ID of the asset which is making this trip.

    The asset will be linked to this trip.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    attributes: object
    """
    `attributes` can be used to store custom information about a trip in
    `key`:`value` format. Use `attributes` to add any useful information or context
    to your trips like the driver name, destination etc.

    Please note that the maximum number of `key`:`value` pairs that can be added to
    an `attributes` object is 100. Also, the overall size of `attributes` object
    should not exceed 65kb.
    """

    custom_id: str
    """Set a unique ID for the new `trip`.

    If not provided, an ID will be automatically generated in UUID format. A valid
    `custom_id` can contain letters, numbers, “-”, & “\\__” only.

    Please note that the ID of a `trip` can not be changed once it is created.
    """

    description: str
    """Add a custom description for the trip."""

    meta_data: object
    """An JSON object to collect additional details about the trip.

    Use this property to add any custom information / context about the trip. The
    input will be passed on to the response as-is and can be used to display useful
    information on, for example, a UI app.
    """

    name: str
    """Specify a name for the trip."""

    stops: Iterable[Stop]
    """
    An array of objects to collect the details about all the stops that need to be
    made before the trip ends. Each object represents one stop.
    """


class Stop(TypedDict, total=False):
    geofence_id: str
    """
    Specify the ID of the geofence indicating the area where the asset needs to make
    a stop, during the trip. Only the IDs of geofences created using
    [NextBillion.ai's Geofence API](https://docs.nextbillion.ai/docs/tracking/api/geofence#create-a-geofence)
    are accepted.
    """

    meta_data: object
    """An JSON object to collect additional details about the stop.

    Use this property to add any custom information / context about the stop. The
    input will be passed on to the response as-is and can be used to display useful
    information on, for example, a UI app.
    """

    name: str
    """Specify a custom name for the stop."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .geofence.geofence import Geofence

__all__ = ["GeofenceRetrieveResponse", "Data"]


class Data(BaseModel):
    geofence: Optional[Geofence] = None
    """An object with details of the geofence."""


class GeofenceRetrieveResponse(BaseModel):
    data: Optional[Data] = None

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["RouteStepGeofenceConfig"]


class RouteStepGeofenceConfig(BaseModel):
    radius: Optional[float] = None
    """Specify the radius of the cicular geofence, in meters.

    Once specified, the service will create a geofence with task's location as the
    center of the circle having the given radius. Valid values for `radius` are
    \\[[10, 5000\\]].
    """

    type: Optional[Literal["circle"]] = None
    """Specify the type of the geofence.

    Currently, `circle` is the only suppoeted value.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .polygon_geojson import PolygonGeojson

__all__ = ["ConsolePreviewResponse", "Data"]


class Data(BaseModel):
    geojson: Optional[PolygonGeojson] = None
    """An object with geoJSON details of the geofence.

    The contents of this object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """


class ConsolePreviewResponse(BaseModel):
    data: Optional[Data] = None

    message: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import isochrone_compute_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.isochrone_compute_response import IsochroneComputeResponse

__all__ = ["IsochroneResource", "AsyncIsochroneResource"]


class IsochroneResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IsochroneResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return IsochroneResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IsochroneResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return IsochroneResourceWithStreamingResponse(self)

    def compute(
        self,
        *,
        contours_meters: int,
        contours_minutes: int,
        coordinates: str,
        key: str,
        contours_colors: str | NotGiven = NOT_GIVEN,
        denoise: float | NotGiven = NOT_GIVEN,
        departure_time: int | NotGiven = NOT_GIVEN,
        generalize: float | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        polygons: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IsochroneComputeResponse:
        """
        The NextBillion.ai Isochrone API computes areas that are reachable within a
        specified amount of time from a location, and returns the reachable regions as
        contours of polygons or lines that you can display on a map.

        Args:
          contours_meters: The distances, in meters, to use for each isochrone contour. You can specify up
              to four contours. Distances must be in increasing order. The maximum distance
              that can be specified is 60000 meters (60 km).

          contours_minutes: The times, in minutes, to use for each isochrone contour. You can specify up to
              four contours. Times must be in increasing order. The maximum time that can be
              specified is 40 minutes.

          coordinates: The coordinates of the location which acts as the starting point for which the
              isochrone lines need to be determined.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          contours_colors: The hex code of the color to fill isochrone contour. When requesting multiple
              contours, it is recommended to provide color codes for each of the requested
              contours, separated by a ",". If no colors are specified, the Isochrone API will
              assign a random color scheme to the output.

          denoise: A floating point value from 0.0 to 1.0 can be used to remove smaller contours.
              The default is 1.0. A value of 1.0 will only return the largest contour for a
              given value. A value of 0.5 drops any contours that are less than half the area
              of the largest contour in the set of contours for that same value.

          departure_time: Use this parameter to set a departure time, expressed as UNIX epoch timestamp in
              seconds, for calculating the isochrone contour. The response will consider the
              typical traffic conditions at the given time and return a contour which can be
              reached under those traffic conditions. Please note that if no input is provided
              for this parameter then the traffic conditions at the time of making the request
              are considered.

          generalize: A positive floating point value, in meters, used as the tolerance for
              Douglas-Peucker generalization. There is no upper bound. If no value is
              specified in the request, the Isochrone API will choose the most optimized
              generalization to use for the request. Note that the generalization of contours
              can lead to self-intersections, as well as intersections of adjacent contours.

          mode: Set which driving mode the service should use to determine the contour. For
              example, if you use "car", the API will return an isochrone contour that a car
              can reach within the specified time or after driving the specified distance.
              Using "truck" will return a contour that a truck can reach after taking into
              account appropriate truck routing restrictions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          polygons: Specify whether to return the contours as GeoJSON polygons (true) or linestrings
              (false, default). When polygons=true, any contour that forms a ring is returned
              as a polygon.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/isochrone/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "contours_meters": contours_meters,
                        "contours_minutes": contours_minutes,
                        "coordinates": coordinates,
                        "key": key,
                        "contours_colors": contours_colors,
                        "denoise": denoise,
                        "departure_time": departure_time,
                        "generalize": generalize,
                        "mode": mode,
                        "polygons": polygons,
                    },
                    isochrone_compute_params.IsochroneComputeParams,
                ),
            ),
            cast_to=IsochroneComputeResponse,
        )


class AsyncIsochroneResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIsochroneResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIsochroneResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIsochroneResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncIsochroneResourceWithStreamingResponse(self)

    async def compute(
        self,
        *,
        contours_meters: int,
        contours_minutes: int,
        coordinates: str,
        key: str,
        contours_colors: str | NotGiven = NOT_GIVEN,
        denoise: float | NotGiven = NOT_GIVEN,
        departure_time: int | NotGiven = NOT_GIVEN,
        generalize: float | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        polygons: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IsochroneComputeResponse:
        """
        The NextBillion.ai Isochrone API computes areas that are reachable within a
        specified amount of time from a location, and returns the reachable regions as
        contours of polygons or lines that you can display on a map.

        Args:
          contours_meters: The distances, in meters, to use for each isochrone contour. You can specify up
              to four contours. Distances must be in increasing order. The maximum distance
              that can be specified is 60000 meters (60 km).

          contours_minutes: The times, in minutes, to use for each isochrone contour. You can specify up to
              four contours. Times must be in increasing order. The maximum time that can be
              specified is 40 minutes.

          coordinates: The coordinates of the location which acts as the starting point for which the
              isochrone lines need to be determined.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          contours_colors: The hex code of the color to fill isochrone contour. When requesting multiple
              contours, it is recommended to provide color codes for each of the requested
              contours, separated by a ",". If no colors are specified, the Isochrone API will
              assign a random color scheme to the output.

          denoise: A floating point value from 0.0 to 1.0 can be used to remove smaller contours.
              The default is 1.0. A value of 1.0 will only return the largest contour for a
              given value. A value of 0.5 drops any contours that are less than half the area
              of the largest contour in the set of contours for that same value.

          departure_time: Use this parameter to set a departure time, expressed as UNIX epoch timestamp in
              seconds, for calculating the isochrone contour. The response will consider the
              typical traffic conditions at the given time and return a contour which can be
              reached under those traffic conditions. Please note that if no input is provided
              for this parameter then the traffic conditions at the time of making the request
              are considered.

          generalize: A positive floating point value, in meters, used as the tolerance for
              Douglas-Peucker generalization. There is no upper bound. If no value is
              specified in the request, the Isochrone API will choose the most optimized
              generalization to use for the request. Note that the generalization of contours
              can lead to self-intersections, as well as intersections of adjacent contours.

          mode: Set which driving mode the service should use to determine the contour. For
              example, if you use "car", the API will return an isochrone contour that a car
              can reach within the specified time or after driving the specified distance.
              Using "truck" will return a contour that a truck can reach after taking into
              account appropriate truck routing restrictions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          polygons: Specify whether to return the contours as GeoJSON polygons (true) or linestrings
              (false, default). When polygons=true, any contour that forms a ring is returned
              as a polygon.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/isochrone/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "contours_meters": contours_meters,
                        "contours_minutes": contours_minutes,
                        "coordinates": coordinates,
                        "key": key,
                        "contours_colors": contours_colors,
                        "denoise": denoise,
                        "departure_time": departure_time,
                        "generalize": generalize,
                        "mode": mode,
                        "polygons": polygons,
                    },
                    isochrone_compute_params.IsochroneComputeParams,
                ),
            ),
            cast_to=IsochroneComputeResponse,
        )


class IsochroneResourceWithRawResponse:
    def __init__(self, isochrone: IsochroneResource) -> None:
        self._isochrone = isochrone

        self.compute = to_raw_response_wrapper(
            isochrone.compute,
        )


class AsyncIsochroneResourceWithRawResponse:
    def __init__(self, isochrone: AsyncIsochroneResource) -> None:
        self._isochrone = isochrone

        self.compute = async_to_raw_response_wrapper(
            isochrone.compute,
        )


class IsochroneResourceWithStreamingResponse:
    def __init__(self, isochrone: IsochroneResource) -> None:
        self._isochrone = isochrone

        self.compute = to_streamed_response_wrapper(
            isochrone.compute,
        )


class AsyncIsochroneResourceWithStreamingResponse:
    def __init__(self, isochrone: AsyncIsochroneResource) -> None:
        self._isochrone = isochrone

        self.compute = async_to_streamed_response_wrapper(
            isochrone.compute,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["GeofenceCreateResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Unique ID of the geofence created.

    It will be the same as `custom_id`, if provided. Else it will be an auto
    generated UUID. Please note this ID cannot be updated.
    """


class GeofenceCreateResponse(BaseModel):
    data: Optional[Data] = None
    """A data object containing the ID of the geofence created."""

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .trip_stop import TripStop
from .asset_details import AssetDetails
from .asset.track_location import TrackLocation

__all__ = ["TripGetSummaryResponse", "Data", "DataTrip"]


class DataTrip(BaseModel):
    id: Optional[str] = None
    """Returns the unique identifier of the trip."""

    asset: Optional[AssetDetails] = None
    """An object with details of the `asset` properties."""

    asset_id: Optional[str] = None
    """
    Returns the ID of the asset linked to the trip when the trip was started or
    updated.
    """

    attributes: Optional[object] = None
    """
    Returns the `attributes` provided for the trip at the time of starting or
    updating it.
    """

    created_at: Optional[int] = None
    """
    Returns the time, expressed as UNIX epoch timestamp in milliseconds, when the
    trip was created.
    """

    description: Optional[str] = None
    """
    Returns the custom description for the trip as provided at the time of starting
    or updating the trip.
    """

    distance: Optional[float] = None
    """Returns the total distance covered during the trip, in meters.

    Please note that this field will be available in the response only if a minimum
    of 3 locations were tracked during the trip.
    """

    duration: Optional[float] = None
    """Returns the total duration elapsed during the trip, in seconds.

    Please note that this field will be available in the response only if a minimum
    of 3 locations were tracked during the trip.
    """

    ended_at: Optional[int] = None
    """
    Returns the time, expressed as UNIX epoch timestamp in milliseconds, when the
    trip ended.
    """

    geometry: Optional[List[str]] = None
    """
    Returns the geometry of the route that was taken during the trip, encoded in
    polyline format. Please note that this field will be available in the response
    only if a minimum of 3 locations were tracked during the trip.
    """

    meta_data: Optional[object] = None
    """
    Returns the metadata containing additional information about the trip as
    provided at the time of starting or updating the trip.
    """

    name: Optional[str] = None
    """
    Returns the name for the trip as provided at the time of starting or updating
    the trip.
    """

    route: Optional[List[TrackLocation]] = None
    """
    An array of object returning the details of the locations tracked for the asset
    during the trip which has ended. Each object represents a single location that
    was tracked.

    Please note that this attribute will not be present in the response if no
    locations were tracked/uploaded during the trip.
    """

    started_at: Optional[int] = None
    """
    Returns the time, expressed as UNIX epoch timestamp in milliseconds, when the
    trip started.
    """

    state: Optional[str] = None
    """Returns the current state of the trip.

    The value will be "active" if the trip is still going on, otherwise the value
    returned would be "ended".
    """

    stops: Optional[List[TripStop]] = None
    """An array of objects returning the details of the stops made during the trip.

    Each object represents a single stop.
    """

    updated_at: Optional[int] = None
    """
    Returns the timeme, expressed as UNIX epoch timestamp in milliseconds, when the
    trip was last updated.
    """


class Data(BaseModel):
    trip: Optional[DataTrip] = None
    """An object containing the returned trip summary."""


class TripGetSummaryResponse(BaseModel):
    data: Optional[Data] = None
    """An container for the trip returned by the service."""

    message: Optional[str] = None
    """Displays the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import snap_to_road_snap_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.snap_to_road_snap_response import SnapToRoadSnapResponse

__all__ = ["SnapToRoadsResource", "AsyncSnapToRoadsResource"]


class SnapToRoadsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SnapToRoadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return SnapToRoadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SnapToRoadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return SnapToRoadsResourceWithStreamingResponse(self)

    def snap(
        self,
        *,
        key: str,
        path: str,
        approaches: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        avoid: Literal["toll", "ferry", "highway", "none"] | NotGiven = NOT_GIVEN,
        geometry: Literal["`polyline`", "`polyline6`", "`geojson`"] | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        option: Literal["`flexible`"] | NotGiven = NOT_GIVEN,
        radiuses: str | NotGiven = NOT_GIVEN,
        road_info: Literal["`max_speed`"] | NotGiven = NOT_GIVEN,
        timestamps: str | NotGiven = NOT_GIVEN,
        tolerate_outlier: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SnapToRoadSnapResponse:
        """
        Nextbillion.ai Snap To Roads API takes a series of locations along a route, and
        returns the new locations on this route that are snapped to the best-matched
        roads where the trip took place. You can set various parameters, such as
        timestamps or radius, to optimize the result.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          path: Pipe-separated list of coordinate points along a path which would be snapped to
              a road.

          approaches: A semicolon-separated list indicating the side of the road from which to
              approach the locations on the snapped route. When set to "unrestricted" a route
              can arrive at the snapped location from either side of the road and when set to
              "curb" the route will arrive at the snapped location on the driving side of the
              region. Please note the number of values provided must be equal to the number of
              coordinate points provided in the "path" parameter. However, you can skip a
              coordinate and show its position in the list with the ";" separator.

          avoid: Setting this will ensure the route avoids ferries, tolls, highways or nothing.
              Multiple values should be separated by a pipe (|). If "none" is provided along
              with other values, an error is returned as a valid route is not feasible. Please
              note that when this parameter is not provided in the input, ferries are set to
              be avoided by default. When this parameter is provided, only the mentioned
              objects are avoided.

          geometry: Sets the output format of the route geometry in the response. Only the
              "polyline" or "polyline6" encoded "geometry" of the snapped path is returned in
              the response depending on the value provided in the input. When "geojson" is
              selected as the input value, "polyline6" encoded geometry of the snapped path is
              returned along with a "geojson" object.

          mode: Set which driving mode the service should use to determine a route. For example,
              if you use "car", the API will return a route that a car can take. Using "truck"
              will return a route a truck can use, taking into account appropriate truck
              routing restrictions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          option: Include this parameter in the request to return segment-wise speed information
              of the route returned in the response.

              Please note that returning speed information is a function of "road_info"
              parameter, which is effective only when "option=flexible". However, the
              resultant route might not contain all the locations provided in "path" input.

          radiuses: Pipe separated radiuses, in meters (m), up to which a coordinate point can be
              snapped. Please note, if no valid road is available within the specified radius,
              the API would snap the points to nearest, most viable road. When using this
              parameter, it is recommended to specify as many radius values as the number of
              points in "path" parameter. If the same number of "radiuses" are not provided,
              the API will use the default radius value of 25 meters for all locations.

          road_info: Use this parameter to receive segment-wise maximum speed information of the
              route in the response. "max_speed" is the only allowed value.

          timestamps: Pipe-separated UNIX epoch timestamp in seconds for each location. If used, the
              number of timestamps must be equal to the number of coordinate points in the
              "path" parameter. The "timestamps" must increase monotonically starting from the
              first timestamp. This means that each subsequent timestamp should either be more
              than or equal to the preceding one.

          tolerate_outlier: Enable it to ignore locations outside the service boundary. When "true", the
              service would ignore "path" coordinates points falling outside the accessible
              area, which otherwise would cause an error when this parameter is "false".

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/snapToRoads/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "path": path,
                        "approaches": approaches,
                        "avoid": avoid,
                        "geometry": geometry,
                        "mode": mode,
                        "option": option,
                        "radiuses": radiuses,
                        "road_info": road_info,
                        "timestamps": timestamps,
                        "tolerate_outlier": tolerate_outlier,
                    },
                    snap_to_road_snap_params.SnapToRoadSnapParams,
                ),
            ),
            cast_to=SnapToRoadSnapResponse,
        )


class AsyncSnapToRoadsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSnapToRoadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSnapToRoadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSnapToRoadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncSnapToRoadsResourceWithStreamingResponse(self)

    async def snap(
        self,
        *,
        key: str,
        path: str,
        approaches: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        avoid: Literal["toll", "ferry", "highway", "none"] | NotGiven = NOT_GIVEN,
        geometry: Literal["`polyline`", "`polyline6`", "`geojson`"] | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        option: Literal["`flexible`"] | NotGiven = NOT_GIVEN,
        radiuses: str | NotGiven = NOT_GIVEN,
        road_info: Literal["`max_speed`"] | NotGiven = NOT_GIVEN,
        timestamps: str | NotGiven = NOT_GIVEN,
        tolerate_outlier: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SnapToRoadSnapResponse:
        """
        Nextbillion.ai Snap To Roads API takes a series of locations along a route, and
        returns the new locations on this route that are snapped to the best-matched
        roads where the trip took place. You can set various parameters, such as
        timestamps or radius, to optimize the result.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          path: Pipe-separated list of coordinate points along a path which would be snapped to
              a road.

          approaches: A semicolon-separated list indicating the side of the road from which to
              approach the locations on the snapped route. When set to "unrestricted" a route
              can arrive at the snapped location from either side of the road and when set to
              "curb" the route will arrive at the snapped location on the driving side of the
              region. Please note the number of values provided must be equal to the number of
              coordinate points provided in the "path" parameter. However, you can skip a
              coordinate and show its position in the list with the ";" separator.

          avoid: Setting this will ensure the route avoids ferries, tolls, highways or nothing.
              Multiple values should be separated by a pipe (|). If "none" is provided along
              with other values, an error is returned as a valid route is not feasible. Please
              note that when this parameter is not provided in the input, ferries are set to
              be avoided by default. When this parameter is provided, only the mentioned
              objects are avoided.

          geometry: Sets the output format of the route geometry in the response. Only the
              "polyline" or "polyline6" encoded "geometry" of the snapped path is returned in
              the response depending on the value provided in the input. When "geojson" is
              selected as the input value, "polyline6" encoded geometry of the snapped path is
              returned along with a "geojson" object.

          mode: Set which driving mode the service should use to determine a route. For example,
              if you use "car", the API will return a route that a car can take. Using "truck"
              will return a route a truck can use, taking into account appropriate truck
              routing restrictions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          option: Include this parameter in the request to return segment-wise speed information
              of the route returned in the response.

              Please note that returning speed information is a function of "road_info"
              parameter, which is effective only when "option=flexible". However, the
              resultant route might not contain all the locations provided in "path" input.

          radiuses: Pipe separated radiuses, in meters (m), up to which a coordinate point can be
              snapped. Please note, if no valid road is available within the specified radius,
              the API would snap the points to nearest, most viable road. When using this
              parameter, it is recommended to specify as many radius values as the number of
              points in "path" parameter. If the same number of "radiuses" are not provided,
              the API will use the default radius value of 25 meters for all locations.

          road_info: Use this parameter to receive segment-wise maximum speed information of the
              route in the response. "max_speed" is the only allowed value.

          timestamps: Pipe-separated UNIX epoch timestamp in seconds for each location. If used, the
              number of timestamps must be equal to the number of coordinate points in the
              "path" parameter. The "timestamps" must increase monotonically starting from the
              first timestamp. This means that each subsequent timestamp should either be more
              than or equal to the preceding one.

          tolerate_outlier: Enable it to ignore locations outside the service boundary. When "true", the
              service would ignore "path" coordinates points falling outside the accessible
              area, which otherwise would cause an error when this parameter is "false".

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/snapToRoads/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "path": path,
                        "approaches": approaches,
                        "avoid": avoid,
                        "geometry": geometry,
                        "mode": mode,
                        "option": option,
                        "radiuses": radiuses,
                        "road_info": road_info,
                        "timestamps": timestamps,
                        "tolerate_outlier": tolerate_outlier,
                    },
                    snap_to_road_snap_params.SnapToRoadSnapParams,
                ),
            ),
            cast_to=SnapToRoadSnapResponse,
        )


class SnapToRoadsResourceWithRawResponse:
    def __init__(self, snap_to_roads: SnapToRoadsResource) -> None:
        self._snap_to_roads = snap_to_roads

        self.snap = to_raw_response_wrapper(
            snap_to_roads.snap,
        )


class AsyncSnapToRoadsResourceWithRawResponse:
    def __init__(self, snap_to_roads: AsyncSnapToRoadsResource) -> None:
        self._snap_to_roads = snap_to_roads

        self.snap = async_to_raw_response_wrapper(
            snap_to_roads.snap,
        )


class SnapToRoadsResourceWithStreamingResponse:
    def __init__(self, snap_to_roads: SnapToRoadsResource) -> None:
        self._snap_to_roads = snap_to_roads

        self.snap = to_streamed_response_wrapper(
            snap_to_roads.snap,
        )


class AsyncSnapToRoadsResourceWithStreamingResponse:
    def __init__(self, snap_to_roads: AsyncSnapToRoadsResource) -> None:
        self._snap_to_roads = snap_to_roads

        self.snap = async_to_streamed_response_wrapper(
            snap_to_roads.snap,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .document_template_content_request_param import DocumentTemplateContentRequestParam

__all__ = ["DocumentTemplateCreateParams"]


class DocumentTemplateCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    content: Required[Iterable[DocumentTemplateContentRequestParam]]
    """A form field that drivers must complete when executing a route step.

    Defines the data structure and validation rules for collecting required
    information during route execution.
    """

    name: Required[str]
    """Specify a name for the document template to be created."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .trip_stop import TripStop
from .asset.track_location import TrackLocation

__all__ = ["TripRetrieveResponse", "Data", "DataTrip"]


class DataTrip(BaseModel):
    id: Optional[str] = None
    """Returns the unique identifier of the trip."""

    asset_id: Optional[str] = None
    """
    Returns the ID of the asset linked to the trip when the trip was started or
    updated.
    """

    attributes: Optional[object] = None
    """
    Returns the `attributes` provided for the trip at the time of starting or
    updating it.
    """

    created_at: Optional[int] = None
    """
    Returns the time, expressed as UNIX epoch timestamp in milliseconds, when the
    trip was created.
    """

    description: Optional[str] = None
    """
    Returns the custom description for the trip as provided at the time of starting
    or updating the trip.
    """

    ended_at: Optional[int] = None
    """
    Returns the time, expressed as UNIX epoch timestamp in milliseconds, when the
    trip ended.
    """

    meta_data: Optional[object] = None
    """
    Returns the metadata containing additional information about the trip as
    provided at the time of starting or updating the trip.
    """

    name: Optional[str] = None
    """
    Returns the name for the trip as provided at the time of starting or updating
    the trip.
    """

    route: Optional[List[TrackLocation]] = None
    """
    An array of object returning the details of the locations tracked for the asset
    during the trip which has ended. Each object represents a single location that
    was tracked.

    Please note that this attribute will not be present in the response if no
    locations were tracked/uploaded during the trip.
    """

    started_at: Optional[int] = None
    """
    Returns the time, expressed as UNIX epoch timestamp in milliseconds, when the
    trip started.
    """

    state: Optional[str] = None
    """Returns the current state of the trip.

    The value will be "active" if the trip is still going on, otherwise the value
    returned would be "ended".
    """

    stops: Optional[List[TripStop]] = None
    """An array of objects returning the details of the stops made during the trip.

    Each object represents a single stop.
    """

    updated_at: Optional[int] = None
    """
    Returns the timeme, expressed as UNIX epoch timestamp in milliseconds, when the
    trip was last updated.
    """


class Data(BaseModel):
    trip: Optional[DataTrip] = None
    """An object containing the returned trip details."""


class TripRetrieveResponse(BaseModel):
    data: Optional[Data] = None
    """An container for the trip returned by the service."""

    message: Optional[str] = None
    """Displays the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["GeocodeBatchCreateParams", "Body"]


class GeocodeBatchCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    body: Required[Iterable[Body]]


_BodyReservedKeywords = TypedDict(
    "_BodyReservedKeywords",
    {
        "in": str,
    },
    total=False,
)


class Body(_BodyReservedKeywords, total=False):
    q: Required[str]
    """Specify the free-text search query.

    Please note that whitespace, urls, email addresses, or other out-of-scope
    queries will yield no results.
    """

    at: str
    """Specify the center of the search context expressed as coordinates.

    Please note that one of "at", "in=circle" or "in=bbox" should be provided for
    relevant results.
    """

    lang: str
    """
    Select the language to be used for result rendering from a list of
    [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
    codes.
    """

    limit: int
    """Maximum number of results to be returned.

    Please note that the minimum value that can be provided is 1 and the maximum
    that can be provided is 100.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .geofence.geofence import Geofence

__all__ = ["GeofenceContainsResponse", "Data", "DataResultList", "DataResultListResult"]


class DataResultListResult(BaseModel):
    contain: Optional[bool] = None
    """`true` when a coordinate point in `locations` is contained by this geofence."""

    location_index: Optional[int] = None
    """Index of the coordinate point in the input `locations`."""


class DataResultList(BaseModel):
    geofence_detail: Optional[Geofence] = None
    """An object with details of the geofence."""

    geofence_id: Optional[str] = None
    """ID of the geofence provided/generated at the time of creating the geofence."""

    result: Optional[List[DataResultListResult]] = None
    """
    An array of objects with results of the contains check for each of the
    coordinate points in `locations` against the geofence represented by
    `geofence_id`.
    """


class Data(BaseModel):
    result_list: Optional[List[DataResultList]] = None
    """
    An array of objects containing each of the geofences provided in the `geofences`
    input. If `geofences` in not provided then the array will return all the
    geofences associated with the `key`
    """


class GeofenceContainsResponse(BaseModel):
    data: Optional[Data] = None

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = [
    "RestrictionsItemListResponse",
    "RestrictionsItemListResponseItem",
    "RestrictionsItemListResponseItemCoordinate",
]


class RestrictionsItemListResponseItemCoordinate(BaseModel):
    lat: Optional[float] = None

    lon: Optional[float] = None


class RestrictionsItemListResponseItem(BaseModel):
    id: float

    area: str

    coordinate: RestrictionsItemListResponseItemCoordinate

    group_id: float

    group_type: Literal["segment", "turn"]

    mode: List[Literal["0w", "1w", "2w", "3w", "4w", "6w"]]

    repeat_on: str

    restriction_type: Literal["closure", "fixedspeed", "maxspeed", "turn", "truck"]

    source: Literal["rrt", "pbf"]

    state: Literal["enabled", "disabled", "deleted"]

    status: Literal["active", "inactive"]


RestrictionsItemListResponse: TypeAlias = List[RestrictionsItemListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .json import (
    JsonResource,
    AsyncJsonResource,
    JsonResourceWithRawResponse,
    AsyncJsonResourceWithRawResponse,
    JsonResourceWithStreamingResponse,
    AsyncJsonResourceWithStreamingResponse,
)
from .distancematrix import (
    DistancematrixResource,
    AsyncDistancematrixResource,
    DistancematrixResourceWithRawResponse,
    AsyncDistancematrixResourceWithRawResponse,
    DistancematrixResourceWithStreamingResponse,
    AsyncDistancematrixResourceWithStreamingResponse,
)

__all__ = [
    "JsonResource",
    "AsyncJsonResource",
    "JsonResourceWithRawResponse",
    "AsyncJsonResourceWithRawResponse",
    "JsonResourceWithStreamingResponse",
    "AsyncJsonResourceWithStreamingResponse",
    "DistancematrixResource",
    "AsyncDistancematrixResource",
    "DistancematrixResourceWithRawResponse",
    "AsyncDistancematrixResourceWithRawResponse",
    "DistancematrixResourceWithStreamingResponse",
    "AsyncDistancematrixResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .monitor import Monitor as Monitor
from .trip_stop import TripStop as TripStop
from .pagination import Pagination as Pagination
from .simple_resp import SimpleResp as SimpleResp
from .asset_details import AssetDetails as AssetDetails
from .metadata_param import MetadataParam as MetadataParam
from .meta_data_param import MetaDataParam as MetaDataParam
from .search_response import SearchResponse as SearchResponse
from .trip_end_params import TripEndParams as TripEndParams
from .asset_bind_params import AssetBindParams as AssetBindParams
from .asset_list_params import AssetListParams as AssetListParams
from .trip_start_params import TripStartParams as TripStartParams
from .asset_track_params import AssetTrackParams as AssetTrackParams
from .trip_delete_params import TripDeleteParams as TripDeleteParams
from .trip_update_params import TripUpdateParams as TripUpdateParams
from .asset_create_params import AssetCreateParams as AssetCreateParams
from .asset_delete_params import AssetDeleteParams as AssetDeleteParams
from .asset_list_response import AssetListResponse as AssetListResponse
from .asset_update_params import AssetUpdateParams as AssetUpdateParams
from .monitor_list_params import MonitorListParams as MonitorListParams
from .search_bound_params import SearchBoundParams as SearchBoundParams
from .trip_start_response import TripStartResponse as TripStartResponse
from .config_update_params import ConfigUpdateParams as ConfigUpdateParams
from .search_around_params import SearchAroundParams as SearchAroundParams
from .trip_retrieve_params import TripRetrieveParams as TripRetrieveParams
from .asset_create_response import AssetCreateResponse as AssetCreateResponse
from .asset_retrieve_params import AssetRetrieveParams as AssetRetrieveParams
from .monitor_create_params import MonitorCreateParams as MonitorCreateParams
from .monitor_delete_params import MonitorDeleteParams as MonitorDeleteParams
from .monitor_list_response import MonitorListResponse as MonitorListResponse
from .monitor_update_params import MonitorUpdateParams as MonitorUpdateParams
from .config_retrieve_params import ConfigRetrieveParams as ConfigRetrieveParams
from .trip_retrieve_response import TripRetrieveResponse as TripRetrieveResponse
from .asset_retrieve_response import AssetRetrieveResponse as AssetRetrieveResponse
from .monitor_create_response import MonitorCreateResponse as MonitorCreateResponse
from .monitor_retrieve_params import MonitorRetrieveParams as MonitorRetrieveParams
from .trip_get_summary_params import TripGetSummaryParams as TripGetSummaryParams
from .config_retrieve_response import ConfigRetrieveResponse as ConfigRetrieveResponse
from .monitor_retrieve_response import MonitorRetrieveResponse as MonitorRetrieveResponse
from .trip_get_summary_response import TripGetSummaryResponse as TripGetSummaryResponse
from .config_test_webhook_params import ConfigTestWebhookParams as ConfigTestWebhookParams
from .config_test_webhook_response import ConfigTestWebhookResponse as ConfigTestWebhookResponse
from .asset_update_attributes_params import AssetUpdateAttributesParams as AssetUpdateAttributesParams
from .namespaced_apikey_create_params import NamespacedApikeyCreateParams as NamespacedApikeyCreateParams
from .namespaced_apikey_delete_params import NamespacedApikeyDeleteParams as NamespacedApikeyDeleteParams
from .namespaced_apikey_create_response import NamespacedApikeyCreateResponse as NamespacedApikeyCreateResponse
from .namespaced_apikey_delete_response import NamespacedApikeyDeleteResponse as NamespacedApikeyDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["BatchRetrieveResponse", "Response"]


class Response(BaseModel):
    response: Optional[object] = None
    """An object returning the routing solution of an individual query.

    The JSON format and structure of the `response` would vary depending on the
    routing endpoint used in each individual query. However, it will be consistent
    with standard response for a given routing endpoint.
    """

    status_code: Optional[int] = None
    """Returns the HTTP status code for the individual routing request.

    See the [API Errors Codes](#api-error-codes) section below for more information.
    """


class BatchRetrieveResponse(BaseModel):
    msg: Optional[str] = None
    """Displays the error message in case of a failed request or operation.

    Please note that this parameter is not returned in the response in case of a
    successful request.
    """

    responses: Optional[List[Response]] = None
    """
    An array of objects returning the results of all the individual routing queries
    specified in the input. Each object represents the solution to an individual
    query in the input.
    """

    status: Optional[str] = None
    """Returns the overall status of the API request.

    Its value will always be one of `success`, `failed`, and `pending`.
    """

    track_id: Optional[str] = None
    """Returns the unique ID of the batch processing task."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["IsochroneComputeParams"]


class IsochroneComputeParams(TypedDict, total=False):
    contours_meters: Required[int]
    """The distances, in meters, to use for each isochrone contour.

    You can specify up to four contours. Distances must be in increasing order. The
    maximum distance that can be specified is 60000 meters (60 km).
    """

    contours_minutes: Required[int]
    """The times, in minutes, to use for each isochrone contour.

    You can specify up to four contours. Times must be in increasing order. The
    maximum time that can be specified is 40 minutes.
    """

    coordinates: Required[str]
    """
    The coordinates of the location which acts as the starting point for which the
    isochrone lines need to be determined.
    """

    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    contours_colors: str
    """The hex code of the color to fill isochrone contour.

    When requesting multiple contours, it is recommended to provide color codes for
    each of the requested contours, separated by a ",". If no colors are specified,
    the Isochrone API will assign a random color scheme to the output.
    """

    denoise: float
    """A floating point value from 0.0 to 1.0 can be used to remove smaller contours.

    The default is 1.0. A value of 1.0 will only return the largest contour for a
    given value. A value of 0.5 drops any contours that are less than half the area
    of the largest contour in the set of contours for that same value.
    """

    departure_time: int
    """
    Use this parameter to set a departure time, expressed as UNIX epoch timestamp in
    seconds, for calculating the isochrone contour. The response will consider the
    typical traffic conditions at the given time and return a contour which can be
    reached under those traffic conditions. Please note that if no input is provided
    for this parameter then the traffic conditions at the time of making the request
    are considered.
    """

    generalize: float
    """
    A positive floating point value, in meters, used as the tolerance for
    Douglas-Peucker generalization. There is no upper bound. If no value is
    specified in the request, the Isochrone API will choose the most optimized
    generalization to use for the request. Note that the generalization of contours
    can lead to self-intersections, as well as intersections of adjacent contours.
    """

    mode: Literal["`car`", "`truck`"]
    """Set which driving mode the service should use to determine the contour.

    For example, if you use "car", the API will return an isochrone contour that a
    car can reach within the specified time or after driving the specified distance.
    Using "truck" will return a contour that a truck can reach after taking into
    account appropriate truck routing restrictions.

    Note: Only the "car" profile is enabled by default. Please note that customized
    profiles (including "truck") might not be available for all regions. Please
    contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
    representative or reach out at
    [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
    additional profiles.
    """

    polygons: bool
    """
    Specify whether to return the contours as GeoJSON polygons (true) or linestrings
    (false, default). When polygons=true, any contour that forms a ring is returned
    as a polygon.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["AssetDetails", "LatestLocation", "LatestLocationLocation"]


class LatestLocationLocation(BaseModel):
    lat: Optional[float] = None
    """Latitude of the tracked location of the `asset`."""

    lon: Optional[float] = None
    """Longitude of the tracked location of the `asset`."""


class LatestLocation(BaseModel):
    accuracy: Optional[float] = None
    """
    If available, this property returns the accuracy of the GPS information received
    at the last tracked location. It is represented as an estimated horizontal
    accuracy radius, in meters, at the 68th percentile confidence level.
    """

    altitude: Optional[float] = None
    """
    If available in the GPS information, this property returns the altitude of the
    `asset` at the last tracked location. It is represented as height, in meters,
    above the WGS84 reference ellipsoid.
    """

    bearing: Optional[float] = None
    """
    If available in the GPS information, this property returns the heading of the
    `asset` calculated from true north in clockwise direction at the last tracked
    location. Please note that the bearing is not affected by the device
    orientation.

    The bearing will always be in the range of [0, 360).
    """

    location: Optional[LatestLocationLocation] = None
    """An object with the coordinates of the last tracked location."""

    speed: Optional[float] = None
    """
    If available in the GPS information, this property returns the speed of the
    `asset`, in meters per second, at the last tracked location.
    """

    timestamp: Optional[int] = None
    """
    A UNIX epoch timestamp in milliseconds, representing the time at which the
    location was tracked.
    """


class AssetDetails(BaseModel):
    id: Optional[str] = None
    """ID of the `asset`.

    This is the same ID that was generated/provided at the time of creating the
    `asset`.
    """

    attributes: Optional[object] = None
    """A string dictionary object containing `attributes` of the `asset`.

    These `attributes` were associated with the `asset` at the time of creating or
    updating it.

    `attributes` can be added to an `asset` using the _Update Asset Attributes_
    method.
    """

    created_at: Optional[int] = None
    """
    A UNIX epoch timestamp in seconds representing the time at which the `asset` was
    created.
    """

    description: Optional[str] = None
    """Description of the `asset`.

    The value would be the same as that provided for the `description` parameter at
    the time of creating or updating the `asset`.
    """

    device_id: Optional[str] = None
    """ID of the `device` that is linked to this asset.

    Please note that there can be multiple `device_id` linked to a single `asset`.
    An empty response is returned if no devices are linked to the `asset`.

    User can link a device to an `asset` using the _Bind Asset to Device_ method.
    """

    latest_location: Optional[LatestLocation] = None
    """An object with details of the last tracked location of the asset."""

    meta_data: Optional[object] = None
    """Any valid json object data.

    Can be used to save customized data. Max size is 65kb.
    """

    name: Optional[str] = None
    """Name of the `asset`.

    The value would be the same as that provided for the `name` parameter at the
    time of creating or updating the `asset`.
    """

    state: Optional[str] = None
    """State of the asset.

    It will be "active" when the asset is in use or available for use, and it will
    be "deleted" in case the asset has been deleted.
    """

    tags: Optional[List[str]] = None
    """
    **This parameter will be deprecated soon! Please move existing `tags` to
    `attributes` parameter.**

    Tags of the asset. These were associated with the `asset` when it was created or
    updated. `tags` can be used for filtering assets in operations like _Get Asset
    List_ and asset **Search** methods. They can also be used for monitoring of
    assets using **Monitor** methods after linking `tags` and `asset`.
    """

    tracked_at: Optional[int] = None
    """
    A UNIX epoch timestamp in seconds representing the last time when the `asset`
    was tracked.
    """

    updated_at: Optional[int] = None
    """
    A UNIX epoch timestamp in seconds representing the time at which the `asset` was
    last updated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .document_template_content_response import DocumentTemplateContentResponse

__all__ = ["DocumentTemplateCreateResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Returns the unique ID of the document template created."""

    content: Optional[List[DocumentTemplateContentResponse]] = None
    """
    An array of objects returning the details of data structures and validation
    rules and other properties of all document fields. Each object represents one
    document field.
    """

    name: Optional[str] = None
    """Returns the name of the document template as specified in the input."""


class DocumentTemplateCreateResponse(BaseModel):
    data: Optional[Data] = None
    """An object returning the details of the document template created."""

    msg: Optional[str] = None
    """Returns the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[int] = None
    """Returns the HTTP response code."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.skynet import config_update_params, config_retrieve_params, config_test_webhook_params
from ...types.skynet.simple_resp import SimpleResp
from ...types.skynet.config_retrieve_response import ConfigRetrieveResponse
from ...types.skynet.config_test_webhook_response import ConfigTestWebhookResponse

__all__ = ["ConfigResource", "AsyncConfigResource"]


class ConfigResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConfigResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ConfigResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConfigResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return ConfigResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigRetrieveResponse:
        """
        Get webhook configuration

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/skynet/config",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    config_retrieve_params.ConfigRetrieveParams,
                ),
            ),
            cast_to=ConfigRetrieveResponse,
        )

    def update(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        webhook: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Update webhook configuration

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          webhook: Use this array to update information about the webhooks. Please note that the
              webhooks will be overwritten every time this method is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._put(
            "/skynet/config",
            body=maybe_transform({"webhook": webhook}, config_update_params.ConfigUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    config_update_params.ConfigUpdateParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    def test_webhook(
        self,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigTestWebhookResponse:
        """
        Test webhook configurations

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/skynet/config/testwebhook",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, config_test_webhook_params.ConfigTestWebhookParams),
            ),
            cast_to=ConfigTestWebhookResponse,
        )


class AsyncConfigResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConfigResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConfigResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConfigResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncConfigResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigRetrieveResponse:
        """
        Get webhook configuration

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/skynet/config",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    config_retrieve_params.ConfigRetrieveParams,
                ),
            ),
            cast_to=ConfigRetrieveResponse,
        )

    async def update(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        webhook: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Update webhook configuration

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          webhook: Use this array to update information about the webhooks. Please note that the
              webhooks will be overwritten every time this method is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._put(
            "/skynet/config",
            body=await async_maybe_transform({"webhook": webhook}, config_update_params.ConfigUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    config_update_params.ConfigUpdateParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    async def test_webhook(
        self,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigTestWebhookResponse:
        """
        Test webhook configurations

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/skynet/config/testwebhook",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, config_test_webhook_params.ConfigTestWebhookParams),
            ),
            cast_to=ConfigTestWebhookResponse,
        )


class ConfigResourceWithRawResponse:
    def __init__(self, config: ConfigResource) -> None:
        self._config = config

        self.retrieve = to_raw_response_wrapper(
            config.retrieve,
        )
        self.update = to_raw_response_wrapper(
            config.update,
        )
        self.test_webhook = to_raw_response_wrapper(
            config.test_webhook,
        )


class AsyncConfigResourceWithRawResponse:
    def __init__(self, config: AsyncConfigResource) -> None:
        self._config = config

        self.retrieve = async_to_raw_response_wrapper(
            config.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            config.update,
        )
        self.test_webhook = async_to_raw_response_wrapper(
            config.test_webhook,
        )


class ConfigResourceWithStreamingResponse:
    def __init__(self, config: ConfigResource) -> None:
        self._config = config

        self.retrieve = to_streamed_response_wrapper(
            config.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            config.update,
        )
        self.test_webhook = to_streamed_response_wrapper(
            config.test_webhook,
        )


class AsyncConfigResourceWithStreamingResponse:
    def __init__(self, config: AsyncConfigResource) -> None:
        self._config = config

        self.retrieve = async_to_streamed_response_wrapper(
            config.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            config.update,
        )
        self.test_webhook = async_to_streamed_response_wrapper(
            config.test_webhook,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .document_template_content_request_param import DocumentTemplateContentRequestParam

__all__ = ["DocumentTemplateUpdateParams"]


class DocumentTemplateUpdateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    content: Iterable[DocumentTemplateContentRequestParam]
    """
    An object to collect the details of form fields to be updated - data structures,
    validation rules. Please note that the details provided here will overwrite any
    existing document fields in the given template.
    """

    name: str
    """Specify the document template name to be updated."""
